@@ -8,37 +8,15 @@
  *                               <zeeshan.ali@nokia.com>
  *         Jens Georg <jensg@openismus.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
- */
-
-/**
- * SECTION:gssdp-client
- * @short_description: SSDP "bus" wrapper.
- *
- * #GSSDPClient wraps the SSDP "bus" as used by both #GSSDPResourceBrowser
- * and #GSSDPResourceGroup.
  */
 
 #define G_LOG_DOMAIN "gssdp-client"
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
-#include <gio/gio.h>
 
+#include <gio/gio.h>
 
 #include "gssdp-client.h"
 #include "gssdp-client-private.h"
@@ -105,6 +83,17 @@ struct _GSSDPClientPrivate {
 
 typedef struct _GSSDPClientPrivate GSSDPClientPrivate;
 
+/**
+ * GSSDPClient:
+ *
+ * A simple SSDP bus handler.
+ *
+ * The [class@GSSDP.Client] will usually be used by the [class@GSSDP.ResourceGroup]
+ * for announcing or the [class@GSSDP.ResourceBrowser] for finding resources on the network.
+ *
+ * A GSSDPClient is required per IP address that you want to use, even if those
+ * belong t the same network device.
+ */
 G_DEFINE_TYPE_EXTENDED (GSSDPClient,
                         gssdp_client,
                         G_TYPE_OBJECT,
@@ -112,7 +101,7 @@ G_DEFINE_TYPE_EXTENDED (GSSDPClient,
                         G_ADD_PRIVATE(GSSDPClient)
                         G_IMPLEMENT_INTERFACE
                                 (G_TYPE_INITABLE,
-                                 gssdp_client_initable_iface_init));
+                                 gssdp_client_initable_iface_init))
 
 struct _GSSDPHeaderField {
         char *name;
@@ -470,7 +459,7 @@ gssdp_client_class_init (GSSDPClientClass *klass)
         object_class->finalize     = gssdp_client_finalize;
 
         /**
-         * GSSDPClient:server-id:
+         * GSSDPClient:server-id:(attributes org.gtk.Property.get=gssdp_client_get_server_id org.gtk.Property.set=gssdp_client_set_server_id):
          *
          * The SSDP server's identifier.
          **/
@@ -487,7 +476,7 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                           G_PARAM_STATIC_BLURB));
 
         /**
-         * GSSDPClient:interface:
+         * GSSDPClient:interface:(attributes org.gtk.Property.get=gssdp_client_get_interface):
          *
          * The name of the network interface this client is associated with.
          * Set to NULL to autodetect.
@@ -507,7 +496,7 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                           G_PARAM_STATIC_BLURB));
 
         /**
-         * GSSDPClient:network:
+         * GSSDPClient:network:(attributes org.gtk.Property.get=gssdp_client_get_network):
          *
          * The network this client is currently connected to. You could set this
          * to anything you want to identify the network this client is
@@ -531,7 +520,7 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                           G_PARAM_STATIC_BLURB));
 
         /**
-         * GSSDPClient:host-ip:
+         * GSSDPClient:host-ip:(attributes org.gtk.Property.get=gssdp_client_get_host_ip):
          *
          * The IP address of the assoicated network interface.
          **/
@@ -550,7 +539,7 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                                       G_PARAM_STATIC_BLURB));
 
         /**
-         * GSSDPClient:host-mask:
+         * GSSDPClient:host-mask:(attributes org.gtk.Property.get=gssdp_client_get_address_mask):
          *
          * The network mask of the assoicated network interface.
          **/
@@ -567,7 +556,7 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                                       G_PARAM_STATIC_STRINGS));
 
         /**
-         * GSSDPClient:active:
+         * GSSDPClient:active:(attributes org.gtk.Property.get=gssdp_client_get_active):
          *
          * Whether this client is active or not (passive). When active
          * (default), the client sends messages on the network, otherwise
@@ -628,17 +617,17 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                          G_PARAM_STATIC_STRINGS));
 
         /**
-         * GSSDPClient:address-family:
+         * GSSDPClient:address-family:(attributes org.gtk.Property.get=gssdp_client_get_family):
          *
          * The IP protocol address family this client works on. When specified
          * during construction without giving a concrete address, it will be
          * used to determine the proper address.
          *
          * If not specified, will contain the currrent address family after
-         * the call to g_initable_init()<!-- -->. Use #G_SOCKET_FAMILY_INVALID
+         * the call to g_initable_init()<!-- -->. Use %G_SOCKET_FAMILY_INVALID
          * to specifiy using the default socket family (legacy IP)
          *
-         * Since: 1.1.1
+         * Since: 1.2.0
          */
         g_object_class_install_property
                 (object_class,
@@ -654,11 +643,11 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                          G_PARAM_STATIC_STRINGS));
 
         /**
-         * GSSDPClient:uda-version:
+         * GSSDPClient:uda-version:(attributes org.gtk.Property.get=gssdp_client_get_uda_version):
          *
          * The UPnP version the client adheres to.
          *
-         * Since: 1.1.1
+         * Since: 1.2.0
          */
         g_object_class_install_property
                 (object_class,
@@ -674,11 +663,11 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                          G_PARAM_STATIC_STRINGS));
 
         /**
-         * GSSDPClient:boot-id:
+         * GSSDPClient:boot-id:(attributes org.gtk.Property.set=gssdp_client_set_boot_id):
          *
          * The value of the BOOTID.UPNP.ORG header
          *
-         * Since 1.1.2
+         * Since 1.2.0
          */
         g_object_class_install_property
                 (object_class,
@@ -695,11 +684,11 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                          G_PARAM_STATIC_STRINGS));
 
         /**
-         * GSSDPClient:config-id:
+         * GSSDPClient:config-id:(attributes org.gtk.Property.set=gssdp_client_set_config_id):
          *
-         * The value of the BOOTID.UPNP.ORG header
+         * The value of the CONFIGID.UPNP.ORG header
          *
-         * Since 1.1.2
+         * Since 1.2.0
          */
         g_object_class_install_property
                 (object_class,
@@ -741,12 +730,18 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                               soup_message_headers_get_type());
 }
 
+
 /**
  * gssdp_client_new:
- * @iface: (allow-none): The name of the network interface, or %NULL for auto-detection.
- * @error: Location to store error, or NULL
+ * @iface: (nullable): The name of the network interface, or %NULL for auto-detection.
+ * @error: (nullable): Location to store error, or %NULL
  *
- * Return value: A new #GSSDPClient object.
+ * Creates a GSSDP client on @iface. GSSDPClient will pick the address it finds
+ * suitable for using.
+ *
+ * Using this utility function, the created client will be using UDA 1.0 and IPv4 only.
+ *
+ * Return value: (nullable): A new #GSSDPClient object.
  **/
 GSSDPClient *
 gssdp_client_new (const char *iface, GError **error)
@@ -760,14 +755,19 @@ gssdp_client_new (const char *iface, GError **error)
 
 /**
  * gssdp_client_new_with_port:
- * @iface: (allow-none): The name of the network interface, or %NULL for
+ * @iface: (nullable): The name of the network interface, or %NULL for
  * auto-detection.
  * @msearch_port: The network port to use for M-SEARCH requests or 0 for
  * random.
  * @error: (allow-none): Location to store error, or %NULL.
  *
- * Return value: A new #GSSDPClient object.
- **/
+ * Creates a GSSDP client on @iface. GSSDPClient will pick the address it finds
+ * suitable for using.
+ *
+ * Using this utility function, the created client will be using UDA 1.0 and IPv4 only.
+ *
+ * Return value: (nullable):  A new #GSSDPClient object or %NULL on error.
+ */
 GSSDPClient *
 gssdp_client_new_with_port (const char *iface,
                             guint16     msearch_port,
@@ -782,12 +782,17 @@ gssdp_client_new_with_port (const char *iface,
 }
 
 /**
- * gssdp_client_set_server_id:
+ * gssdp_client_set_server_id:(attributes org.gtk.Method.set_property=server-id):
  * @client: A #GSSDPClient
  * @server_id: The server ID
  *
- * Sets the server ID of @client to @server_id.
- **/
+ * Sets the server ID of @client to @server_id. This string is used as the
+ * "Server:" identification header for SSDP discovery and response packets
+ * and "User-Agent" header for searches.
+ *
+ * By default, GSSDP will generate a header conforming to the requirements
+ * defined in the UDA documents: OS/Version UPnP/Version GSSDP/Version.
+ */
 void
 gssdp_client_set_server_id (GSSDPClient *client,
                             const char  *server_id)
@@ -806,7 +811,7 @@ gssdp_client_set_server_id (GSSDPClient *client,
 }
 
 /**
- * gssdp_client_get_server_id:
+ * gssdp_client_get_server_id:(attributes org.gtk.Method.get_property=server-id):
  * @client: A #GSSDPClient
  *
  * Return value: The server ID.
@@ -823,7 +828,7 @@ gssdp_client_get_server_id (GSSDPClient *client)
 }
 
 /**
- * gssdp_client_get_interface:
+ * gssdp_client_get_interface:(attributes org.gtk.Method.get_property=interface):
  * @client: A #GSSDPClient
  *
  * Get the name of the network interface associated to @client.
@@ -843,7 +848,7 @@ gssdp_client_get_interface (GSSDPClient *client)
 }
 
 /**
- * gssdp_client_get_host_ip:
+ * gssdp_client_get_host_ip:(attributes org.gtk.Method.get_property=host-ip):
  * @client: A #GSSDPClient
  *
  * Get the IP address we advertise ourselves as using.
@@ -866,7 +871,7 @@ gssdp_client_get_host_ip (GSSDPClient *client)
 }
 
 /**
- * gssdp_client_set_network:
+ * gssdp_client_set_network:(attributes org.gtk.Method.set_property=network):
  * @client: A #GSSDPClient
  * @network: The string identifying the network
  *
@@ -894,6 +899,15 @@ gssdp_client_set_network (GSSDPClient *client,
  * @client: A #GSSDPClient
  * @ip_address: The host to add to the cache
  * @user_agent: User agent ot the host to add
+ *
+ * Add @user_agent for @ip_address.
+ *
+ * Each [class@GSSDP.Client] maintains a mapping of addresses
+ * (MAC on systems that support it, IP addresses otherwise) to User Agents.
+ *
+ * This information can be used in higher layers to get an User-Agent for
+ * devices that do not set the User-Agent header in their SOAP requests.
+ *
  **/
 void
 gssdp_client_add_cache_entry (GSSDPClient  *client,
@@ -922,7 +936,9 @@ gssdp_client_add_cache_entry (GSSDPClient  *client,
  * @client: A #GSSDPClient
  * @ip_address: IP address to guess the user-agent for
  *
- * Returns: (transfer none): The user-agent cached for this IP, %NULL if none
+ * Try to get a User-Agent for @ip_address.
+ *
+ * Returns: (transfer none): The User-Agent cached for this IP, %NULL if none
  * is cached.
  **/
 const char *
@@ -953,13 +969,14 @@ gssdp_client_guess_user_agent (GSSDPClient *client,
 }
 
 /**
- * gssdp_client_get_network:
- * @client: A #GSSDPClient
+ * gssdp_client_get_network:(attributes org.gtk.Method.get_property=network):
+ * @client: A [class@GSSDP.Client]
  *
- * Get the network this client is associated with.
+ * Get the network identifier of the client. See [property@GSSDP.Client:network]
+ * for  details.
  *
  * Return value: The network identification. This string should not be freed.
- **/
+ */
 const char *
 gssdp_client_get_network (GSSDPClient *client)
 {
@@ -972,9 +989,11 @@ gssdp_client_get_network (GSSDPClient *client)
 }
 
 /**
- * gssdp_client_get_active:
+ * gssdp_client_get_active:(attributes org.gtk.Method.get_property=active):
  * @client: A #GSSDPClient
  *
+ * Get the current state of the client. See [property@GSSDP.Client:active] for details.
+ *
  * Return value: %TRUE if @client is active, %FALSE otherwise.
  **/
 gboolean
@@ -1020,11 +1039,12 @@ append_header_fields (GList *headers, const gchar *message)
  * gssdp_client_append_header:
  * @client: A #GSSDPClient
  * @name: Header name
- * @value: Header value
+ * @value: (allow-none): Header value
  *
- * Adds a header field to the message sent by this @client. It is intended to
- * be used by clients requiring vendor specific header fields. (If there is an
- * existing header with name name , then this creates a second one).
+ * Adds a header field to the messages sent by this @client. It is intended to
+ * be used by clients requiring vendor specific header fields.
+ *
+ * If there is an existing header with @name it will append another one.
  **/
 void
 gssdp_client_append_header (GSSDPClient *client,
@@ -1036,6 +1056,7 @@ gssdp_client_append_header (GSSDPClient *client,
 
         g_return_if_fail (GSSDP_IS_CLIENT (client));
         g_return_if_fail (name != NULL);
+        g_return_if_fail (value != NULL);
 
         priv = gssdp_client_get_instance_private (client);
 
@@ -1050,7 +1071,7 @@ gssdp_client_append_header (GSSDPClient *client,
  * @client: A #GSSDPClient
  * @name: Header name
  *
- * Removes @name from the list of headers . If there are multiple values for
+ * Removes @name from the list of headers. If there are multiple values for
  * @name, they are all removed.
  **/
 void
@@ -1065,8 +1086,7 @@ gssdp_client_remove_header (GSSDPClient *client,
 
         priv = gssdp_client_get_instance_private (client);
         l = priv->headers;
-        while (l != NULL)
-        {
+        while (l != NULL) {
                 GList *next = l->next;
                 GSSDPHeaderField *header = l->data;
 
@@ -1106,13 +1126,14 @@ gssdp_client_clear_headers (GSSDPClient *client)
                 }
                 l = next;
         }
-
 }
 
 /**
  * gssdp_client_get_address:
  * @client: A #GSSDPClient
  *
+ * The IP address this client works on.
+ *
  * Returns: (transfer full): The #GInetAddress this client works on
  **/
 GInetAddress *
@@ -1140,7 +1161,7 @@ gssdp_client_get_index (GSSDPClient *client)
 }
 
 /**
- * gssdp_client_get_family:
+ * gssdp_client_get_family:(attributes org.gtk.Method.get_property=address-family):
  * @client: A #GSSDPClient
  *
  * Returns: IP protocol version (%G_SOCKET_FAMILY_IPV4 or G_SOCKET_FAMILY_IPV6)
@@ -1156,7 +1177,7 @@ gssdp_client_get_family (GSSDPClient *client)
 }
 
 /**
- * gssdp_client_get_address_mask:
+ * gssdp_client_get_address_mask:(attributes org.gtk.Method.get_property=host-mask):
  * @client: A #GSSDPClient
  *
  * Since: 1.2.3
@@ -1174,7 +1195,7 @@ gssdp_client_get_address_mask (GSSDPClient *client)
 }
 
 /**
- * gssdp_client_get_uda_version:
+ * gssdp_client_get_uda_version:(attributes org.gtk.Method.get_property=uda-version):
  * @client: A #GSSDPClient
  *
  * Returns: the UDA protocol version this client adheres to
@@ -1189,7 +1210,7 @@ gssdp_client_get_uda_version  (GSSDPClient *client)
 }
 
 /**
- * gssdp_client_set_boot_id:
+ * gssdp_client_set_boot_id:(attributes org.gtk.Method.set_property=boot-id):
  * @client: A #GSSDPClient
  * @boot_id: The new boot-id for the client
  *
@@ -1221,7 +1242,7 @@ gssdp_client_set_boot_id (GSSDPClient *client, gint32 boot_id)
 }
 
 /**
- * gssdp_client_set_config_id:
+ * gssdp_client_set_config_id:(attributes org.gtk.Method.set_property=config-id):
  * @client: A #GSSDPClient
  * @config_id: The new config-id for the client
  *
@@ -1247,6 +1268,34 @@ gssdp_client_set_config_id (GSSDPClient *client, gint32 config_id)
 
 }
 
+/**
+ * gssdp_client_can_reach:
+ * @client: A #GSSDPClient
+ * @address: A #GInetSocketAddress of the target. The port part of the address may be 0
+ *
+ * Check if the peer at @address is reachable using this @client.
+ *
+ * Since: 1.2.4
+ * Returns: %TRUE if considered reachable, %FALSE otherwise.
+ */
+gboolean
+gssdp_client_can_reach (GSSDPClient *client, GInetSocketAddress *address)
+{
+        g_return_val_if_fail (GSSDP_IS_CLIENT (client), FALSE);
+        g_return_val_if_fail (G_IS_INET_SOCKET_ADDRESS (address), FALSE);
+
+        GSSDPClientPrivate *priv = gssdp_client_get_instance_private (client);
+
+        GInetAddress *addr = g_inet_socket_address_get_address (address);
+        if (g_inet_address_get_is_link_local (addr) &&
+            g_inet_address_get_family (addr) == G_SOCKET_FAMILY_IPV6) {
+                return g_inet_socket_address_get_scope_id (address) ==
+                       priv->device.index;
+        }
+
+        return g_inet_address_mask_matches (priv->device.host_mask, addr);
+}
+
 /**
  * _gssdp_client_send_message:
  * @client: A #GSSDPClient
@@ -1558,19 +1607,29 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
                                 msg = GSSDP_PKTINFO6_MESSAGE (messages[i]);
                                 msg_ifindex = gssdp_pktinfo6_message_get_ifindex (msg);
                                 local_addr = gssdp_pktinfo6_message_get_local_addr (msg);
-                        } else
+                        } else {
                                 continue;
+                        }
 
                         /* message needs to be on correct interface or on
                          * loopback (as kernel can be smart and route things
                          * there even if sent to another network) */
-                        if (!((msg_ifindex == priv->device.index ||
-                               msg_ifindex == LOOPBACK_IFINDEX) &&
-                              (g_inet_address_equal (local_addr,
-                                                     priv->device.host_addr) ||
-                               g_inet_address_equal (local_addr, group_addr)))) {
-                                goto out;
-                        } else {
+                        if (g_inet_address_equal (local_addr, group_addr)) {
+                                // This is a multicast packet. If the index is not our index, ignore
+                                if (msg_ifindex != priv->device.index) {
+                                        goto out;
+                                }
+                                break;
+                        }
+
+                        if (g_inet_address_equal (local_addr,
+                                                  priv->device.host_addr)) {
+                                // This is a "normal" packet. We can receive those
+
+                                if (msg_ifindex != priv->device.index &&
+                                    msg_ifindex != LOOPBACK_IFINDEX) {
+                                        goto out;
+                                }
                                 break;
                         }
                 }
@@ -1582,14 +1641,9 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
          * on this socket from a particular interface but AFAIK that is not
          * possible, at least not in a portable way.
          */
-        {
-                GInetAddress *inet_address;
-                GInetSocketAddress *sockaddr;
 
-                sockaddr = G_INET_SOCKET_ADDRESS (address);
-                inet_address = g_inet_socket_address_get_address (sockaddr);
-                if (!g_inet_address_mask_matches (priv->device.host_mask, inet_address))
-                        goto out;
+        if (!gssdp_client_can_reach (client, G_INET_SOCKET_ADDRESS(address))) {
+                goto out;
         }
 #endif
 
@@ -1845,7 +1899,7 @@ init_network_info (GSSDPClient *client, GError **error)
 
         g_debug ("Created SSDP client 0x%p", client);
         g_debug ("  iface_name : %s", priv->device.iface_name);
-        g_debug ("  host_ip    : %s", priv->device.host_ip);
+        g_debug ("  host_ip    : %s", gssdp_client_get_host_ip (client));
         g_debug ("  server_id  : %s", priv->server_id);
         g_debug ("  network    : %s", priv->device.network);
         g_debug ("  host_addr  : 0x%p", priv->device.host_addr);
