@@ -184,8 +184,8 @@ gssdp_net_mac_lookup (GSSDPNetworkDevice *device, const char *ip_address)
 
                         msg = NLMSG_DATA (header);
 
-                        rtattr = IFA_RTA (msg);
-                        rtattr_len = IFA_PAYLOAD (header);
+                        rtattr = RTM_RTA (msg);
+                        rtattr_len = RTM_PAYLOAD (header);
 
                         while (RT_ATTR_OK (rtattr, rtattr_len)) {
                                 if (rtattr->rta_type == NDA_DST) {
@@ -524,8 +524,16 @@ gssdp_net_get_host_ip (GSSDPNetworkDevice *device)
                 if (!equal)
                         continue;
 
-                device->host_mask = get_netmask (ifa->ifa_addr,
-                                                 ifa->ifa_netmask);
+                if (device->host_mask != NULL &&
+                    !g_inet_address_mask_matches (device->host_mask,
+                                                  device->host_addr)) {
+                        g_clear_object (&device->host_mask);
+                }
+
+                if (device->host_mask == NULL) {
+                        device->host_mask =
+                                get_netmask (ifa->ifa_addr, ifa->ifa_netmask);
+                }
 
                 if (device->iface_name == NULL)
                         device->iface_name = g_strdup (ifa->ifa_name);
